﻿using UnityEngine;
using UnityEngine.UI;
using System.Collections.Generic;
using System.Linq;
using UnityEngine.SceneManagement;

public class GameManager : MonoBehaviour {

	protected class QueuedFloatingText {
		public HexCard card;
		public int income;
		public int growth;
	}

	public GameObject gameOverWindow;

	public GameObject timeLimitWindow;


	static public GameManager _current;
	static public GameManager current {
		get {
			if(_current == null) {
				_current = GameObject.FindObjectOfType<GameManager>();
			}

			return _current;
		}
	}

	bool keepPlaying = false;

	int _turnsLeft = 24;
	public int turnsLeft {
		get { return _turnsLeft; }
	}

	int _money = 15;
	public int money {
		get { return _money; }
	}
	int money_change = 0;

	float _shareValue = 1f;
	public float shareValue {
		get { return _shareValue; }
	}
	float share_change = 0;

	public Text txtCash;
	public Text txtShareValue;
	public Text txtTurnsLeft;

	Dictionary<HexCard, int> hexMoneyChange;
	Dictionary<HexCard, int> hexShareChange;
	List<QueuedFloatingText> queuedFloatingText;

	public GameObject floatingTextPrefab;



	// Use this for initialization
	void Start () {
		hexMoneyChange = new Dictionary<HexCard, int>();
		hexShareChange = new Dictionary<HexCard, int>();
		queuedFloatingText = new List<QueuedFloatingText>();

		StatsManager.current.Reset();

		UpdateTexts();
	}

	float floatingTextCooldown = 0.1f;

	// Update is called once per frame
	void Update () {
		if(queuedFloatingText.Count > 0) {
			floatingTextCooldown -= Time.deltaTime;
			if(floatingTextCooldown <= 0) {
				floatingTextCooldown = 0.1f;

				QueuedFloatingText q = queuedFloatingText[queuedFloatingText.Count-1];
				queuedFloatingText.RemoveAt(queuedFloatingText.Count-1);

				GameObject floatGO = (GameObject)Instantiate(
					floatingTextPrefab, 
					CardManager.current.CoordToWorld(q.card.x, q.card.y) + new Vector2(0, -0.4f), 
					Quaternion.identity);
				floatGO.GetComponent<FloatingText>().income = q.income;
				floatGO.GetComponent<FloatingText>().growth = q.growth;
			}
		}

	}

	public void UpdateTexts() {
		_shareValue = Mathf.Max(_shareValue, 0.01f);

		txtCash.text = "Cash: $"+money+"m " + 
			( money_change ==0 ? "" : 
				( money_change >=0 ? "(<color=green>+" : "(<color=red>" ) + (money_change) + "</color>)"
			);

		txtShareValue.text = "Stock Price: $"+
			shareValue.ToString("0.##") + 
			( share_change == 0 ? "" :
				(share_change >= 0 ?
				("<color=green>▲"+ share_change.ToString("0.##") +"</color>") :
				("<color=red>▼"+ share_change.ToString("0.##") +"</color>")
				)
			);

		if(keepPlaying) {
			txtTurnsLeft.text = "Unlimited Time!";	
		}
		else {
			txtTurnsLeft.text = "Months Left: "+turnsLeft;
		}


		money_change = 0;
		share_change = 0;

		List<HexCard> changedCards = new List<HexCard>();
		foreach(HexCard card in hexMoneyChange.Keys) {
			changedCards.Add(card);
		}
		foreach(HexCard card in hexShareChange.Keys) {
			if(changedCards.Contains(card) == false) 
				changedCards.Add(card);
		}

		queuedFloatingText.Clear();
		foreach(HexCard card in changedCards) {
			QueuedFloatingText q = new QueuedFloatingText();
			q.card = card;
			q.income = hexMoneyChange.ContainsKey(card) ? hexMoneyChange[card] : 0;
			q.growth = hexShareChange.ContainsKey(card) ? hexShareChange[card] : 0;
			queuedFloatingText.Add(q);
		}
		queuedFloatingText.Sort( (a, b) => (a.card.x+a.card.y).CompareTo(b.card.x+b.card.y) );
		hexMoneyChange.Clear();
		hexShareChange.Clear();

		StatsManager.current.cash = money;
		StatsManager.current.share_price = shareValue;

		if(keepPlaying==false && turnsLeft < 5) {
			timeLimitWindow.SetActive(true);
		}
		else {
			timeLimitWindow.SetActive(false);
		}

		if(IsGameOver()) {
			// FIXME
			//gameOverWindow.SetActive(true);
			GameObject.FindObjectOfType<SwipeTransistion>().doAnimation = true;
		}
	}

	public bool IsGameOver() {
		if(keepPlaying)
			return false;

		if(turnsLeft <= 0) {
			return true;
		}
		return false;
	}

	public void ChangeMoney(HexCard c, int amt, bool countsForChange=true) {
		//Debug.Log("ChangeMoney -- " + c.name + " -- " + amt);
		_money += amt;

		if(countsForChange) {
			money_change += amt;

			StatsManager.current.revenue += amt;

			if(c != null) {
				if(hexMoneyChange.ContainsKey(c)) {
					hexMoneyChange[c] += amt;
				}
				else {
					hexMoneyChange[c] = amt;
				}
			}
		}
	}

	public void ChangeShareValue(HexCard c, float amt) {
		//Debug.Log("ChangeShareValue -- " + c.name + " -- " + amt);
		_shareValue += amt;
		share_change += amt;

		if(c != null) {
			if(hexShareChange.ContainsKey(c)) {
				hexShareChange[c] += (int)amt;
			}
			else {
				hexShareChange[c] = (int)amt;
			}
		}

	}

	public void ChangeTurnsLeft(HexCard c, int amt) {
		if(c != null) {
			//Debug.Log("ChangeTurnsLeft -- " + c.name + " -- " + amt);
		}

		_turnsLeft += amt;
	}

	public void GoToEndScreen() {
		SceneManager.LoadScene( "_SCENE_END_" );
	}

	public void KeepPlaying() {
		keepPlaying = true;
		GameObject.FindObjectOfType<SwipeTransistion>().KeepPlaying();
		txtTurnsLeft.text = "Unlimited Turns!";	
		Tooltip.current.gameObject.SetActive(false);
	}


}
